@@ -30,10 +30,30 @@
 #include "gdk-pixbuf.h"
 #include "gdk-pixbuf-private.h"
 
-static void gdk_pixbuf_class_init  (GdkPixbufClass *klass);
-static void gdk_pixbuf_finalize    (GObject        *object);
+static void gdk_pixbuf_class_init   (GdkPixbufClass *klass);
+static void gdk_pixbuf_finalize     (GObject        *object);
+static void gdk_pixbuf_set_property (GObject        *object,
+				     guint           prop_id,
+				     const GValue   *value,
+				     GParamSpec     *pspec);
+static void gdk_pixbuf_get_property (GObject        *object,
+				     guint           prop_id,
+				     GValue         *value,
+				     GParamSpec     *pspec);
 
 
+enum 
+{
+  PROP_0,
+  PROP_COLORSPACE,
+  PROP_N_CHANNELS,
+  PROP_HAS_ALPHA,
+  PROP_BITS_PER_SAMPLE,
+  PROP_WIDTH,
+  PROP_HEIGHT,
+  PROP_ROWSTRIDE,
+  PROP_PIXELS
+};
 
 static gpointer parent_class;
 
@@ -71,6 +91,91 @@ gdk_pixbuf_class_init (GdkPixbufClass *klass)
         parent_class = g_type_class_peek_parent (klass);
         
         object_class->finalize = gdk_pixbuf_finalize;
+        object_class->set_property = gdk_pixbuf_set_property;
+        object_class->get_property = gdk_pixbuf_get_property;
+        
+        g_object_class_install_property (object_class,
+                                         PROP_N_CHANNELS,
+                                         g_param_spec_int ("n_channels",
+                                                           P_("Number of Channels"),
+                                                           P_("The number of samples per pixel"),
+                                                           0,
+                                                           G_MAXINT,
+                                                           3,
+                                                           G_PARAM_READWRITE |
+                                                           G_PARAM_CONSTRUCT_ONLY));
+
+        g_object_class_install_property (object_class,
+                                         PROP_COLORSPACE,
+                                         g_param_spec_enum ("colorspace",
+                                                            P_("Colorspace"),
+                                                            P_("The colorspace in which the samples are interpreted"),
+                                                            GDK_TYPE_COLORSPACE,
+                                                            GDK_COLORSPACE_RGB,
+                                                            
+                                                            G_PARAM_READWRITE |
+                                                            G_PARAM_CONSTRUCT_ONLY));
+
+        g_object_class_install_property (object_class,
+                                         PROP_HAS_ALPHA,
+                                         g_param_spec_boolean ("has_alpha",
+                                                               P_("Has Alpha"),
+                                                               P_("Whether the pixbuf has an alpha channel"),
+                                                               FALSE,
+                                                               G_PARAM_READWRITE |
+                                                               G_PARAM_CONSTRUCT_ONLY));
+
+        g_object_class_install_property (object_class,
+                                         PROP_BITS_PER_SAMPLE,
+                                         g_param_spec_int ("bits_per_sample",
+                                                           P_("Bits per Sample"),
+                                                           P_("The number of bits per sample"),
+                                                           1,
+                                                           16,
+                                                           8,
+                                                           G_PARAM_READWRITE |
+                                                           G_PARAM_CONSTRUCT_ONLY));
+
+        g_object_class_install_property (object_class,
+                                         PROP_WIDTH,
+                                         g_param_spec_int ("width",
+                                                           P_("Width"),
+                                                           P_("The number of columns of the pixbuf"),
+                                                           1,
+                                                           G_MAXINT,
+                                                           1,
+                                                           G_PARAM_READWRITE |
+                                                           G_PARAM_CONSTRUCT_ONLY));
+
+        g_object_class_install_property (object_class,
+                                         PROP_HEIGHT,
+                                         g_param_spec_int ("height",
+                                                           P_("Height"),
+                                                           P_("The number of rows of the pixbuf"),
+                                                           1,
+                                                           G_MAXINT,
+                                                           1,
+                                                           G_PARAM_READWRITE |
+                                                           G_PARAM_CONSTRUCT_ONLY));
+
+        g_object_class_install_property (object_class,
+                                         PROP_ROWSTRIDE,
+                                         g_param_spec_int ("rowstride",
+                                                           P_("Rowstride"),
+                                                           P_("The number of bytes between the start of a row and the start of the next row"),
+                                                           1,
+                                                           G_MAXINT,
+                                                           1,
+                                                           G_PARAM_READWRITE |
+                                                           G_PARAM_CONSTRUCT_ONLY));
+
+        g_object_class_install_property (object_class,
+                                         PROP_PIXELS,
+                                         g_param_spec_pointer ("pixels",
+                                                               P_("Pixels"),
+                                                               P_("A pointer to the pixel data of the pixbuf"),
+                                                               G_PARAM_READWRITE |
+                                                               G_PARAM_CONSTRUCT_ONLY));
 }
 
 static void
@@ -89,28 +194,30 @@ gdk_pixbuf_finalize (GObject *object)
  * gdk_pixbuf_ref:
  * @pixbuf: A pixbuf.
  *
- * Adds a reference to a pixbuf. Deprecated; use g_object_ref().
+ * Adds a reference to a pixbuf. 
  *
  * Return value: The same as the @pixbuf argument.
+ *
+ * Deprecated: Use g_object_ref().
  **/
 GdkPixbuf *
 gdk_pixbuf_ref (GdkPixbuf *pixbuf)
 {
-        return (GdkPixbuf *) g_object_ref (G_OBJECT(pixbuf));
+        return (GdkPixbuf *) g_object_ref (pixbuf);
 }
 
 /**
  * gdk_pixbuf_unref:
  * @pixbuf: A pixbuf.
  *
- * Removes a reference from a pixbuf. Deprecated; use
- * g_object_unref().
+ * Removes a reference from a pixbuf. 
  *
+ * Deprecated: Use g_object_unref().
  **/
 void
 gdk_pixbuf_unref (GdkPixbuf *pixbuf)
 {
-        g_object_unref (G_OBJECT (pixbuf));
+        g_object_unref (pixbuf);
 }
 
 
@@ -130,16 +237,19 @@ free_buffer (guchar *pixels, gpointer data)
  * @width: Width of image in pixels.
  * @height: Height of image in pixels.
  *
- * Creates a new #GdkPixbuf structure and allocates a buffer for it.  The buffer
- * has an optimal rowstride.  Note that the buffer is not cleared; you will have
- * to fill it completely yourself.
+ * Creates a new #GdkPixbuf structure and allocates a buffer for it.  The 
+ * buffer has an optimal rowstride.  Note that the buffer is not cleared;
+ * you will have to fill it completely yourself.
  *
  * Return value: A newly-created #GdkPixbuf with a reference count of 1, or 
  * %NULL if not enough memory could be allocated for the image buffer.
  **/
 GdkPixbuf *
-gdk_pixbuf_new (GdkColorspace colorspace, gboolean has_alpha, int bits_per_sample,
-		int width, int height)
+gdk_pixbuf_new (GdkColorspace colorspace, 
+                gboolean      has_alpha,
+                int           bits_per_sample,
+                int           width,
+                int           height)
 {
 	guchar *buf;
 	int channels;
@@ -317,7 +427,7 @@ gdk_pixbuf_get_n_channels (const GdkPixbuf *pixbuf)
 gboolean
 gdk_pixbuf_get_has_alpha (const GdkPixbuf *pixbuf)
 {
-	g_return_val_if_fail (pixbuf != NULL, -1);
+	g_return_val_if_fail (pixbuf != NULL, FALSE);
 
 	return pixbuf->has_alpha ? TRUE : FALSE;
 }
@@ -344,7 +454,9 @@ gdk_pixbuf_get_bits_per_sample (const GdkPixbuf *pixbuf)
  *
  * Queries a pointer to the pixel data of a pixbuf.
  *
- * Return value: A pointer to the pixbuf's pixel data.
+ * Return value: A pointer to the pixbuf's pixel data.  Please see <xref linkend="image-data"/>
+ * for information about how the pixel data is stored in
+ * memory.
  **/
 guchar *
 gdk_pixbuf_get_pixels (const GdkPixbuf *pixbuf)
@@ -390,9 +502,10 @@ gdk_pixbuf_get_height (const GdkPixbuf *pixbuf)
  * gdk_pixbuf_get_rowstride:
  * @pixbuf: A pixbuf.
  *
- * Queries the rowstride of a pixbuf, which is the number of bytes between rows.
+ * Queries the rowstride of a pixbuf, which is the number of bytes between the start of a row
+ * and the start of the next row.
  *
- * Return value: Number of bytes between rows.
+ * Return value: Distance between row starts.
  **/
 int
 gdk_pixbuf_get_rowstride (const GdkPixbuf *pixbuf)
@@ -425,7 +538,8 @@ gdk_pixbuf_error_quark (void)
 /**
  * gdk_pixbuf_fill:
  * @pixbuf: a #GdkPixbuf
- * @pixel: RGBA pixel to clear to (0xffffffff is opaque white, 0x00000000 transparent black)
+ * @pixel: RGBA pixel to clear to
+ *         (0xffffffff is opaque white, 0x00000000 transparent black)
  *
  * Clears a pixbuf to the given RGBA value, converting the RGBA value into
  * the pixbuf's pixel format. The alpha will be ignored if the pixbuf
@@ -437,15 +551,14 @@ gdk_pixbuf_fill (GdkPixbuf *pixbuf,
                  guint32    pixel)
 {
         guchar *pixels;
-        gboolean all_the_same = FALSE;
         guint r, g, b, a;
         guchar *p;
-        gint n;
+        guint w, h;
 
         g_return_if_fail (GDK_IS_PIXBUF (pixbuf));
 
         if (pixbuf->width == 0 || pixbuf->height == 0)
-          return;
+                return;
 
         pixels = pixbuf->pixels;
 
@@ -454,44 +567,35 @@ gdk_pixbuf_fill (GdkPixbuf *pixbuf,
         b = (pixel & 0x0000ff00) >> 8;
         a = (pixel & 0x000000ff);
 
-        if (r == g && g == b) {
-                if (!pixbuf->has_alpha)
-                        all_the_same = TRUE;
-                else
-                        all_the_same = (r == a);
-        }
+        h = pixbuf->height;
         
-        if (all_the_same) {
-                if (pixbuf->has_alpha)
-                        memset (pixels, r, pixbuf->width * 4);
-                else
-                        memset (pixels, r, pixbuf->width * 3);
-
-        } else {
-                guchar  c[4];
-
-                c[0] = r; c[1] = g; c[2] = b; c[3] = a;
-                
+        while (h--) {
+                w = pixbuf->width;
                 p = pixels;
-                n = pixbuf->width;
-                if (pixbuf->has_alpha) {
-                        do {
-                                memcpy (p, c, 4);
-                                p += 4;
-                        } while (--n);
-                } else {
-                        do {
-                                memcpy (p, c, 3);
-                                p += 3;
-                        } while (--n);
-                }
 
-        }
-        p = pixels;
-        n = pixbuf->height - 1;
-        while (n--) {
-                p += pixbuf->rowstride;
-                memcpy (p, pixels, pixbuf->width * pixbuf->n_channels);
+                switch (pixbuf->n_channels) {
+                case 3:
+                        while (w--) {
+                                p[0] = r;
+                                p[1] = g;
+                                p[2] = b;
+                                p += 3;
+                        }
+                        break;
+                case 4:
+                        while (w--) {
+                                p[0] = r;
+                                p[1] = g;
+                                p[2] = b;
+                                p[3] = a;
+                                p += 4;
+                        }
+                        break;
+                default:
+                        break;
+                }
+                
+                pixels += pixbuf->rowstride;
         }
 }
 
@@ -541,6 +645,8 @@ gdk_pixbuf_get_option (GdkPixbuf   *pixbuf,
  * ignored and %FALSE is returned.
  *
  * Return value: %TRUE on success.
+ *
+ * Since: 2.2
  **/
 gboolean
 gdk_pixbuf_set_option (GdkPixbuf   *pixbuf,
@@ -581,6 +687,86 @@ gdk_pixbuf_set_option (GdkPixbuf   *pixbuf,
         return TRUE;
 }
 
+static void
+gdk_pixbuf_set_property (GObject         *object,
+			 guint            prop_id,
+			 const GValue    *value,
+			 GParamSpec      *pspec)
+{
+  GdkPixbuf *pixbuf = GDK_PIXBUF (object);
+
+  switch (prop_id)
+          {
+          case PROP_COLORSPACE:
+                  pixbuf->colorspace = g_value_get_enum (value);
+                  break;
+          case PROP_N_CHANNELS:
+                  pixbuf->n_channels = g_value_get_int (value);
+                  break;
+          case PROP_HAS_ALPHA:
+                  pixbuf->has_alpha = g_value_get_boolean (value);
+                  break;
+          case PROP_BITS_PER_SAMPLE:
+                  pixbuf->bits_per_sample = g_value_get_int (value);
+                  break;
+          case PROP_WIDTH:
+                  pixbuf->width = g_value_get_int (value);
+                  break;
+          case PROP_HEIGHT:
+                  pixbuf->height = g_value_get_int (value);
+                  break;
+          case PROP_ROWSTRIDE:
+                  pixbuf->rowstride = g_value_get_int (value);
+                  break;
+          case PROP_PIXELS:
+                  pixbuf->pixels = (guchar *) g_value_get_pointer (value);
+                  break;
+          default:
+                  G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+                  break;
+          }
+}
+
+static void
+gdk_pixbuf_get_property (GObject         *object,
+			 guint            prop_id,
+			 GValue          *value,
+			 GParamSpec      *pspec)
+{
+  GdkPixbuf *pixbuf = GDK_PIXBUF (object);
+  
+  switch (prop_id)
+          {
+          case PROP_COLORSPACE:
+                  g_value_set_enum (value, gdk_pixbuf_get_colorspace (pixbuf));
+                  break;
+          case PROP_N_CHANNELS:
+                  g_value_set_int (value, gdk_pixbuf_get_n_channels (pixbuf));
+                  break;
+          case PROP_HAS_ALPHA:
+                  g_value_set_boolean (value, gdk_pixbuf_get_has_alpha (pixbuf));
+                  break;
+          case PROP_BITS_PER_SAMPLE:
+                  g_value_set_int (value, gdk_pixbuf_get_bits_per_sample (pixbuf));
+                  break;
+          case PROP_WIDTH:
+                  g_value_set_int (value, gdk_pixbuf_get_width (pixbuf));
+                  break;
+          case PROP_HEIGHT:
+                  g_value_set_int (value, gdk_pixbuf_get_height (pixbuf));
+                  break;
+          case PROP_ROWSTRIDE:
+                  g_value_set_int (value, gdk_pixbuf_get_rowstride (pixbuf));
+                  break;
+          case PROP_PIXELS:
+                  g_value_set_pointer (value, gdk_pixbuf_get_pixels (pixbuf));
+                  break;
+          default:
+                  G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+                  break;
+          }
+}
+
 
 
 /* Include the marshallers */
