@@ -77,7 +77,8 @@ typedef enum {
   GDK_DEBUG_IMAGE	  = 1 << 9,
   GDK_DEBUG_INPUT	  = 1 <<10,
   GDK_DEBUG_CURSOR	  = 1 <<11,
-  GDK_DEBUG_MULTIHEAD	  = 1 <<12
+  GDK_DEBUG_MULTIHEAD	  = 1 <<12,
+  GDK_DEBUG_XINERAMA	  = 1 <<13
 } GdkDebugFlag;
 
 #ifndef GDK_DISABLE_DEPRECATED
@@ -166,6 +167,8 @@ extern GDestroyNotify _gdk_event_notify;
 
 extern GSList    *_gdk_displays;
 extern gchar     *_gdk_display_name;
+extern gint       _gdk_screen_number;
+extern gchar     *_gdk_display_arg_name;
 
 void      _gdk_events_queue  (GdkDisplay *display);
 GdkEvent* _gdk_event_unqueue (GdkDisplay *display);
@@ -207,6 +210,10 @@ GdkImage *_gdk_drawable_copy_to_image (GdkDrawable  *drawable,
 				       gint          width,
 				       gint          height);
 
+/* GC caching */
+GdkGC *_gdk_drawable_get_scratch_gc (GdkDrawable *drawable,
+				     gboolean     graphics_exposures);
+
 /*************************************
  * Interfaces used by windowing code *
  *************************************/
@@ -236,6 +243,9 @@ void     _gdk_windowing_init                    (gint       *argc,
                                                  gchar    ***argv);
 void     _gdk_windowing_set_default_display     (GdkDisplay *display);
 
+gchar *_gdk_windowing_substitute_screen_number (const gchar *display_name,
+					        gint         screen_number);
+
 void     _gdk_windowing_window_get_offsets      (GdkWindow  *window,
 						 gint       *x_offset,
 						 gint       *y_offset);
