@@ -52,6 +52,7 @@ enum
   PROP_MINIMUM_KEY_LENGTH
 };
 
+#define GTK_ENTRY_COMPLETION_GET_PRIVATE(obj)(G_TYPE_INSTANCE_GET_PRIVATE ((obj), GTK_TYPE_ENTRY_COMPLETION, GtkEntryCompletionPrivate))
 
 static void     gtk_entry_completion_class_init          (GtkEntryCompletionClass *klass);
 static void     gtk_entry_completion_cell_layout_init    (GtkCellLayoutIface      *iface);
@@ -117,6 +118,7 @@ static void     gtk_entry_completion_action_data_func    (GtkTreeViewColumn
                                                           gpointer                 data);
 
 
+static GObjectClass *parent_class = NULL;
 static guint entry_completion_signals[LAST_SIGNAL] = { 0 };
 
 
@@ -164,6 +166,7 @@ gtk_entry_completion_class_init (GtkEntryCompletionClass *klass)
 {
   GObjectClass *object_class;
 
+  parent_class = g_type_class_peek_parent (klass);
   object_class = (GObjectClass *)klass;
 
   object_class->set_property = gtk_entry_completion_set_property;
@@ -383,6 +386,8 @@ gtk_entry_completion_finalize (GObject *object)
 
   if (completion->priv->popup_window)
     gtk_widget_destroy (completion->priv->popup_window);
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 /* implement cell layout interface */
@@ -506,15 +511,18 @@ gtk_entry_completion_default_completion_func (GtkEntryCompletion *completion,
                       completion->priv->text_column, &item,
                       -1);
 
-  normalized_string = g_utf8_normalize (item, -1, G_NORMALIZE_ALL);
-  case_normalized_string = g_utf8_casefold (normalized_string, -1);
-
-  if (!strncmp (key, case_normalized_string, strlen (key)))
-      ret = TRUE;
-
-  g_free (item);
-  g_free (normalized_string);
-  g_free (case_normalized_string);
+  if (item != NULL)
+    {
+      normalized_string = g_utf8_normalize (item, -1, G_NORMALIZE_ALL);
+      case_normalized_string = g_utf8_casefold (normalized_string, -1);
+      
+      if (!strncmp (key, case_normalized_string, strlen (key)))
+	ret = TRUE;
+      
+      g_free (item);
+      g_free (normalized_string);
+      g_free (case_normalized_string);
+    }
 
   return ret;
 }
@@ -687,6 +695,8 @@ gtk_entry_completion_action_data_func (GtkTreeViewColumn *tree_column,
                   "markup", NULL,
                   "text", string,
                   NULL);
+
+  g_free (string);
 }
 
 static void
@@ -989,7 +999,7 @@ gtk_entry_completion_delete_action (GtkEntryCompletion *completion,
  * @completion: A #GtkEntryCompletion.
  * @column: The column in the model of @completion to get strings from.
  *
- * Conviencefunction for setting up the most used case of this code: a
+ * Convience function for setting up the most used case of this code: a
  * completion list with just strings. This function will set up @completion
  * to have a list displaying all (and just) strings in the completion list,
  * and to get those strings from @column in the model of @completion.
