@@ -22,6 +22,7 @@
 
 #include "gtkfilesystemmodel.h"
 #include "gtkfilesystem.h"
+#include "gtkintl.h"
 #include "gtktreemodel.h"
 
 typedef struct _GtkFileSystemModelClass GtkFileSystemModelClass;
@@ -44,6 +45,7 @@ struct _GtkFileSystemModel
   GtkFileInfoType types;
   FileModelNode  *roots;
   GtkFileFolder  *root_folder;
+  GtkFilePath    *root_path;
 
   GtkFileSystemModelFilter filter_func;
   gpointer filter_data;
@@ -57,6 +59,7 @@ struct _GtkFileSystemModel
   guint show_folders : 1;
   guint show_files : 1;
   guint folders_only : 1;
+  guint has_editable : 1;
 };
 
 struct _FileModelNode
@@ -142,8 +145,10 @@ static void               file_model_node_clear        (GtkFileSystemModel *mode
 static FileModelNode *    file_model_node_get_children (GtkFileSystemModel *model,
 							FileModelNode      *node);
 
+#if 0
 static void roots_changed_callback (GtkFileSystem      *file_system,
 				    GtkFileSystemModel *model);
+#endif
 				    
 static void deleted_callback       (GtkFileFolder *folder,
 				    FileModelNode *node);
@@ -169,6 +174,8 @@ static void root_files_removed_callback (GtkFileFolder      *folder,
 					 GSList             *paths,
 					 GtkFileSystemModel *model);
 
+static GObjectClass *parent_class = NULL;
+
 GType
 _gtk_file_system_model_get_type (void)
 {
@@ -211,7 +218,9 @@ static void
 gtk_file_system_model_class_init (GtkFileSystemModelClass *class)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (class);
-  
+
+  parent_class = g_type_class_peek_parent (class);
+
   gobject_class->finalize = gtk_file_system_model_finalize;
 }
 
@@ -246,17 +255,26 @@ static void
 gtk_file_system_model_finalize (GObject *object)
 {
   GtkFileSystemModel *model = GTK_FILE_SYSTEM_MODEL (object);
-  FileModelNode *children;
+  FileModelNode *children, *next;
 
   if (model->root_folder)
     g_object_unref (model->root_folder);
 
+  if (model->root_path)
+    gtk_file_path_free (model->root_path);
+
+  if (model->file_system)
+    g_object_unref (model->file_system);
+
   children = model->roots;
   while (children)
     {
+      next = children->next;
       file_model_node_free (children);
-      children = children->next;
+      children = next;
     }
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
 /*
@@ -367,18 +385,31 @@ gtk_file_system_model_get_value (GtkTreeModel *tree_model,
 {
   GtkFileSystemModel *model = GTK_FILE_SYSTEM_MODEL (tree_model);
   FileModelNode *node = iter->user_data;
+  const GtkFileInfo *info;
   
   switch (column)
     {
     case GTK_FILE_SYSTEM_MODEL_INFO:
+      if (model->has_editable && node == model->roots)
+	info = NULL;
+      else
+	info = file_model_node_get_info (model, node);
+
       g_value_init (value, GTK_TYPE_FILE_INFO);
-      g_value_set_boxed (value, file_model_node_get_info (model, node));
+      g_value_set_boxed (value, info);
       break;
     case GTK_FILE_SYSTEM_MODEL_DISPLAY_NAME:
       {
-	const GtkFileInfo *info = file_model_node_get_info (model, node);
 	g_value_init (value, G_TYPE_STRING);
-	g_value_set_string (value, gtk_file_info_get_display_name (info));
+
+	if (model->has_editable && node == model->roots)
+	  g_value_set_string (value, "");
+	else
+	  {
+	    const GtkFileInfo *info = file_model_node_get_info (model, node);
+
+	    g_value_set_string (value, gtk_file_info_get_display_name (info));
+	  }
       }
       break;
     default:
@@ -537,9 +568,7 @@ gtk_file_system_model_unref_node (GtkTreeModel *tree_model,
 /**
  * _gtk_file_system_model_new:
  * @file_system: an object implementing #GtkFileSystem
- * @root_path: the path of root of the file system to display,
- *            or %NULL to display starting from the
- *            root or roots of the fielsystem.
+ * @root_path: the path of root of the file system to display
  * @max_depth: the maximum depth from the children of @root_path
  *             or the roots of the file system to display in
  *             the file selector). A depth of 0 displays
@@ -569,6 +598,7 @@ _gtk_file_system_model_new (GtkFileSystem     *file_system,
   GSList *tmp_list;
 
   g_return_val_if_fail (GTK_IS_FILE_SYSTEM (file_system), NULL);
+  g_return_val_if_fail (root_path != NULL, NULL);
 
   model = g_object_new (GTK_TYPE_FILE_SYSTEM_MODEL, NULL);
   model->file_system = g_object_ref (file_system);
@@ -581,7 +611,8 @@ _gtk_file_system_model_new (GtkFileSystem     *file_system,
   if (root_path)
     {
       GSList *child_paths;
-      
+
+      model->root_path = gtk_file_path_copy (root_path);
       model->root_folder = gtk_file_system_get_folder (file_system, root_path,
 						       model->types,
 						       NULL);   /* NULL-GError */
@@ -603,12 +634,14 @@ _gtk_file_system_model_new (GtkFileSystem     *file_system,
 				   G_CALLBACK (root_files_removed_callback), model, 0);
 	}
     }
+#if 0
   else
     {
       roots = gtk_file_system_list_roots (file_system);
       g_signal_connect_object (file_system, "roots-changed",
 			       G_CALLBACK (roots_changed_callback), model, 0);
     }
+#endif
 
   roots = gtk_file_paths_sort (roots);
   
@@ -779,13 +812,22 @@ _gtk_file_system_model_set_show_files (GtkFileSystemModel *model,
  *   is owned by @model and must not be modified or freed.
  *   If you want to save the information for later use,
  *   you must make a copy, since the structure may be
- *   freed on later changes to the file system.
+ *   freed on later changes to the file system.  If you have
+ *   called _gtk_file_system_model_add_editable() and the @iter
+ *   corresponds to the row that this function returned, the
+ *   return value will be NULL.
  **/
 const GtkFileInfo *
 _gtk_file_system_model_get_info (GtkFileSystemModel *model,
 				 GtkTreeIter        *iter)
 {
-  return file_model_node_get_info (model, iter->user_data);
+  FileModelNode *node;
+
+  node = iter->user_data;
+  if (model->has_editable && node == model->roots)
+    return NULL;
+  else
+    return file_model_node_get_info (model, node);
 }
 
 /**
@@ -806,6 +848,9 @@ _gtk_file_system_model_get_path (GtkFileSystemModel *model,
 {
   FileModelNode *node = iter->user_data;
 
+  if (model->has_editable && node == model->roots)
+    return NULL;
+
   if (node->is_dummy)
     return node->parent->path;
   else
@@ -857,16 +902,14 @@ find_and_ref_path (GtkFileSystemModel  *model,
   FileModelNode *child_node;
   GtkFileFolder *folder;
 
-  if (!gtk_file_system_get_parent (model->file_system, path, &parent_path, NULL))
+  if (gtk_file_path_compare (path, model->root_path) == 0
+      || !gtk_file_system_get_parent (model->file_system, path, &parent_path, NULL))
     return NULL;
 
   if (parent_path)
     {
       parent_node = find_and_ref_path (model, parent_path, cleanups);
       gtk_file_path_free (parent_path);
-
-      if (!parent_node)
-	return NULL;
     }
   else
     parent_node = NULL;
@@ -978,6 +1021,71 @@ _gtk_file_system_model_path_do (GtkFileSystemModel       *model,
   return node != NULL;
 }
 
+/**
+ * _gtk_file_system_model_add_editable:
+ * @model: a #GtkFileSystemModel
+ * @iter: Location to return the iter corresponding to the editable row
+ * 
+ * Adds an "empty" row at the beginning of the model.  This does not refer to
+ * any file, but is a temporary placeholder for a file name that the user will
+ * type when a corresponding cell is made editable.  When your code is done
+ * using this temporary row, call _gtk_file_system_model_remove_editable().
+ **/
+void
+_gtk_file_system_model_add_editable (GtkFileSystemModel *model, GtkTreeIter *iter)
+{
+  FileModelNode *node;
+  GtkTreePath *path;
+
+  g_return_if_fail (!model->has_editable);
+
+  model->has_editable = TRUE;
+
+  node = file_model_node_new (model, NULL);
+  node->is_visible = TRUE;
+
+  node->next = model->roots;
+  model->roots = node;
+
+  file_model_node_ref (node);
+
+  path = gtk_tree_path_new ();
+  gtk_tree_path_append_index (path, 0);
+  iter->user_data = node;
+
+  gtk_tree_model_row_inserted (GTK_TREE_MODEL (model), path, iter);
+
+  gtk_tree_path_free (path);
+}
+
+/**
+ * _gtk_file_system_model_remove_editable:
+ * @model: a #GtkFileSystemModel
+ * 
+ * Removes the "empty" row at the beginning of the model that was
+ * created with _gtk_file_system_model_add_editable().  You should call
+ * this function when your code is finished editing this temporary row.
+ **/
+void
+_gtk_file_system_model_remove_editable (GtkFileSystemModel *model)
+{
+  GtkTreePath *path;
+
+  g_return_if_fail (model->has_editable);
+
+  model->has_editable = FALSE;
+  file_model_node_unref (model, model->roots);
+
+  model->roots = model->roots->next;
+
+  path = gtk_tree_path_new ();
+  gtk_tree_path_append_index (path, 0);
+
+  gtk_tree_model_row_deleted (GTK_TREE_MODEL (model), path);
+
+  gtk_tree_path_free (path);
+}
+
 static FileModelNode *
 file_model_node_new (GtkFileSystemModel *model,
 		     const GtkFilePath  *path)
@@ -1013,7 +1121,7 @@ file_model_node_get_info (GtkFileSystemModel *model,
       if (node->is_dummy)
 	{
 	  node->info = gtk_file_info_new ();
-	  gtk_file_info_set_display_name (node->info, "(Empty)");
+	  gtk_file_info_set_display_name (node->info, _("(Empty)"));
 	}
       else if (node->parent || model->root_folder)
 	{
@@ -1021,6 +1129,7 @@ file_model_node_get_info (GtkFileSystemModel *model,
 						 node->path,
 						 NULL);	/* NULL-GError */
 	}
+#if 0
       else
 	{
 	  node->info = gtk_file_system_get_root_info (model->file_system,
@@ -1028,6 +1137,7 @@ file_model_node_get_info (GtkFileSystemModel *model,
 						      model->types,
 						      NULL);  /* NULL-GError */
 	}
+#endif
     }
 
   return node->info;
@@ -1555,6 +1665,7 @@ do_files_removed (GtkFileSystemModel *model,
   g_slist_free (sorted_paths);
 }
 
+#if 0
 static void
 roots_changed_callback (GtkFileSystem      *file_system,
 			GtkFileSystemModel *model)
@@ -1651,6 +1762,7 @@ roots_changed_callback (GtkFileSystem      *file_system,
   g_slist_free (new_roots);
   gtk_tree_path_free (path);
 }
+#endif
 
 static void
 deleted_callback (GtkFileFolder      *folder,
