@@ -20,6 +20,8 @@
 
 #include "gtkfilesystem.h"
 #include "gtkfilesystemunix.h"
+#include "gtkicontheme.h"
+#include "gtkintl.h"
 
 #define XDG_PREFIX _gtk_xdg
 #include "xdgmime/xdgmime.h"
@@ -29,6 +31,10 @@
 #include <sys/stat.h>
 #include <sys/types.h>
 #include <unistd.h>
+#include <stdio.h>
+
+#define BOOKMARKS_FILENAME ".gtk-bookmarks"
+#define BOOKMARKS_TMP_FILENAME ".gtk-bookmarks-XXXXXX"
 
 typedef struct _GtkFileSystemUnixClass GtkFileSystemUnixClass;
 
@@ -44,8 +50,25 @@ struct _GtkFileSystemUnixClass
 struct _GtkFileSystemUnix
 {
   GObject parent_instance;
+
+  GHashTable *folder_hash;
 };
 
+/* Icon type, supplemented by MIME type
+ */
+typedef enum {
+  ICON_NONE,
+  ICON_REGULAR,	/* Use mime type for icon */
+  ICON_BLOCK_DEVICE,
+  ICON_BROKEN_SYMBOLIC_LINK,
+  ICON_CHARACTER_DEVICE,
+  ICON_DIRECTORY,
+  ICON_EXECUTABLE,
+  ICON_FIFO,
+  ICON_SOCKET
+} IconType;
+
+
 #define GTK_TYPE_FILE_FOLDER_UNIX             (gtk_file_folder_unix_get_type ())
 #define GTK_FILE_FOLDER_UNIX(obj)             (G_TYPE_CHECK_INSTANCE_CAST ((obj), GTK_TYPE_FILE_FOLDER_UNIX, GtkFileFolderUnix))
 #define GTK_IS_FILE_FOLDER_UNIX(obj)          (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GTK_TYPE_FILE_FOLDER_UNIX))
@@ -65,6 +88,7 @@ struct _GtkFileFolderUnix
 {
   GObject parent_instance;
 
+  GtkFileSystemUnix *system_unix;
   GtkFileInfoType types;
   gchar *filename;
 };
@@ -77,11 +101,10 @@ static void gtk_file_system_unix_iface_init   (GtkFileSystemIface     *iface);
 static void gtk_file_system_unix_init         (GtkFileSystemUnix      *impl);
 static void gtk_file_system_unix_finalize     (GObject                *object);
 
-static GSList *       gtk_file_system_unix_list_roots    (GtkFileSystem      *file_system);
-static GtkFileInfo *  gtk_file_system_unix_get_root_info (GtkFileSystem      *file_system,
-							  const GtkFilePath  *path,
-							  GtkFileInfoType     types,
-							  GError            **error);
+static GSList *             gtk_file_system_unix_list_volumes        (GtkFileSystem     *file_system);
+static GtkFileSystemVolume *gtk_file_system_unix_get_volume_for_path (GtkFileSystem     *file_system,
+								      const GtkFilePath *path);
+
 static GtkFileFolder *gtk_file_system_unix_get_folder    (GtkFileSystem      *file_system,
 							  const GtkFilePath  *path,
 							  GtkFileInfoType     types,
@@ -89,6 +112,24 @@ static GtkFileFolder *gtk_file_system_unix_get_folder    (GtkFileSystem      *fi
 static gboolean       gtk_file_system_unix_create_folder (GtkFileSystem      *file_system,
 							  const GtkFilePath  *path,
 							  GError            **error);
+
+static void         gtk_file_system_unix_volume_free             (GtkFileSystem       *file_system,
+								  GtkFileSystemVolume *volume);
+static GtkFilePath *gtk_file_system_unix_volume_get_base_path    (GtkFileSystem       *file_system,
+								  GtkFileSystemVolume *volume);
+static gboolean     gtk_file_system_unix_volume_get_is_mounted   (GtkFileSystem       *file_system,
+								  GtkFileSystemVolume *volume);
+static gboolean     gtk_file_system_unix_volume_mount            (GtkFileSystem       *file_system,
+								  GtkFileSystemVolume *volume,
+								  GError             **error);
+static gchar *      gtk_file_system_unix_volume_get_display_name (GtkFileSystem       *file_system,
+								  GtkFileSystemVolume *volume);
+static GdkPixbuf *  gtk_file_system_unix_volume_render_icon      (GtkFileSystem        *file_system,
+								  GtkFileSystemVolume  *volume,
+								  GtkWidget            *widget,
+								  gint                  pixel_size,
+								  GError              **error);
+
 static gboolean       gtk_file_system_unix_get_parent    (GtkFileSystem      *file_system,
 							  const GtkFilePath  *path,
 							  GtkFilePath       **parent,
@@ -113,6 +154,12 @@ static GtkFilePath *gtk_file_system_unix_uri_to_path      (GtkFileSystem     *fi
 static GtkFilePath *gtk_file_system_unix_filename_to_path (GtkFileSystem     *file_system,
 							   const gchar       *filename);
 
+static GdkPixbuf *gtk_file_system_unix_render_icon (GtkFileSystem     *file_system,
+						    const GtkFilePath *path,
+						    GtkWidget         *widget,
+						    gint               pixel_size,
+						    GError           **error);
+
 static gboolean gtk_file_system_unix_add_bookmark    (GtkFileSystem     *file_system,
 						      const GtkFilePath *path,
 						      GError           **error);
@@ -134,7 +181,6 @@ static gboolean     gtk_file_folder_unix_list_children (GtkFileFolder  *folder,
 							GSList        **children,
 							GError        **error);
 
-static gchar *      filename_from_path (const GtkFilePath *path);
 static GtkFilePath *filename_to_path   (const gchar       *filename);
 
 static gboolean     filename_is_root  (const char       *filename);
@@ -164,7 +210,7 @@ gtk_file_system_unix_get_type (void)
 	0,		/* n_preallocs */
 	(GInstanceInitFunc) gtk_file_system_unix_init,
       };
-      
+
       static const GInterfaceInfo file_system_info =
       {
 	(GInterfaceInitFunc) gtk_file_system_unix_iface_init, /* interface_init */
@@ -185,11 +231,11 @@ gtk_file_system_unix_get_type (void)
 
 /**
  * gtk_file_system_unix_new:
- * 
+ *
  * Creates a new #GtkFileSystemUnix object. #GtkFileSystemUnix
  * implements the #GtkFileSystem interface with direct access to
  * the filesystem using Unix/Linux API calls
- * 
+ *
  * Return value: the new #GtkFileSystemUnix object
  **/
 GtkFileSystem *
@@ -204,17 +250,23 @@ gtk_file_system_unix_class_init (GtkFileSystemUnixClass *class)
   GObjectClass *gobject_class = G_OBJECT_CLASS (class);
 
   system_parent_class = g_type_class_peek_parent (class);
-  
+
   gobject_class->finalize = gtk_file_system_unix_finalize;
 }
 
 static void
 gtk_file_system_unix_iface_init   (GtkFileSystemIface *iface)
 {
-  iface->list_roots = gtk_file_system_unix_list_roots;
+  iface->list_volumes = gtk_file_system_unix_list_volumes;
+  iface->get_volume_for_path = gtk_file_system_unix_get_volume_for_path;
   iface->get_folder = gtk_file_system_unix_get_folder;
-  iface->get_root_info = gtk_file_system_unix_get_root_info;
   iface->create_folder = gtk_file_system_unix_create_folder;
+  iface->volume_free = gtk_file_system_unix_volume_free;
+  iface->volume_get_base_path = gtk_file_system_unix_volume_get_base_path;
+  iface->volume_get_is_mounted = gtk_file_system_unix_volume_get_is_mounted;
+  iface->volume_mount = gtk_file_system_unix_volume_mount;
+  iface->volume_get_display_name = gtk_file_system_unix_volume_get_display_name;
+  iface->volume_render_icon = gtk_file_system_unix_volume_render_icon;
   iface->get_parent = gtk_file_system_unix_get_parent;
   iface->make_path = gtk_file_system_unix_make_path;
   iface->parse = gtk_file_system_unix_parse;
@@ -222,6 +274,7 @@ gtk_file_system_unix_iface_init   (GtkFileSystemIface *iface)
   iface->path_to_filename = gtk_file_system_unix_path_to_filename;
   iface->uri_to_path = gtk_file_system_unix_uri_to_path;
   iface->filename_to_path = gtk_file_system_unix_filename_to_path;
+  iface->render_icon = gtk_file_system_unix_render_icon;
   iface->add_bookmark = gtk_file_system_unix_add_bookmark;
   iface->remove_bookmark = gtk_file_system_unix_remove_bookmark;
   iface->list_bookmarks = gtk_file_system_unix_list_bookmarks;
@@ -230,31 +283,40 @@ gtk_file_system_unix_iface_init   (GtkFileSystemIface *iface)
 static void
 gtk_file_system_unix_init (GtkFileSystemUnix *system_unix)
 {
+  system_unix->folder_hash = g_hash_table_new (g_str_hash, g_str_equal);
 }
 
 static void
 gtk_file_system_unix_finalize (GObject *object)
 {
+  GtkFileSystemUnix *system_unix;
+
+  system_unix = GTK_FILE_SYSTEM_UNIX (object);
+
+  /* FIXME: assert that the hash is empty? */
+  g_hash_table_destroy (system_unix->folder_hash);
+
   system_parent_class->finalize (object);
 }
 
-static GSList *
-gtk_file_system_unix_list_roots (GtkFileSystem *file_system)
+/* Returns our single root volume */
+static GtkFileSystemVolume *
+get_root_volume (void)
 {
-  return g_slist_append (NULL, gtk_file_path_new_dup ("/"));
+  return (GtkFileSystemVolume *) gtk_file_path_new_dup ("/");
 }
 
-static GtkFileInfo *
-gtk_file_system_unix_get_root_info (GtkFileSystem    *file_system,
-				    const GtkFilePath      *path,
-				    GtkFileInfoType   types,
-				    GError          **error)
+static GSList *
+gtk_file_system_unix_list_volumes (GtkFileSystem *file_system)
 {
-  const gchar *filename = gtk_file_path_get_string (path);
-  
-  g_return_val_if_fail (strcmp (filename, "/") == 0, NULL);
+  return g_slist_append (NULL, get_root_volume ());
+}
 
-  return filename_get_info ("/", types, error);
+static GtkFileSystemVolume *
+gtk_file_system_unix_get_volume_for_path (GtkFileSystem     *file_system,
+					  const GtkFilePath *path)
+{
+  return get_root_volume ();
 }
 
 static GtkFileFolder *
@@ -263,17 +325,34 @@ gtk_file_system_unix_get_folder (GtkFileSystem     *file_system,
 				 GtkFileInfoType    types,
 				 GError           **error)
 {
+  GtkFileSystemUnix *system_unix;
   GtkFileFolderUnix *folder_unix;
-  gchar *filename;
+  const char *filename;
 
-  filename = filename_from_path (path);
+  system_unix = GTK_FILE_SYSTEM_UNIX (file_system);
+
+  filename = gtk_file_path_get_string (path);
   g_return_val_if_fail (filename != NULL, NULL);
+  g_return_val_if_fail (g_path_is_absolute (filename), NULL);
 
-  folder_unix = g_object_new (GTK_TYPE_FILE_FOLDER_UNIX, NULL);
-  folder_unix->filename = filename;
-  folder_unix->types = types;
+  folder_unix = g_hash_table_lookup (system_unix->folder_hash, filename);
 
-  return GTK_FILE_FOLDER (folder_unix);
+  if (folder_unix)
+    {
+      folder_unix->types |= types;
+      return g_object_ref (folder_unix);
+    }
+  else
+    {
+      folder_unix = g_object_new (GTK_TYPE_FILE_FOLDER_UNIX, NULL);
+      folder_unix->system_unix = system_unix;
+      folder_unix->filename = g_strdup (filename);
+      folder_unix->types = types;
+
+      g_hash_table_insert (system_unix->folder_hash, folder_unix->filename, folder_unix);
+
+      return GTK_FILE_FOLDER (folder_unix);
+    }
 }
 
 static gboolean
@@ -281,29 +360,228 @@ gtk_file_system_unix_create_folder (GtkFileSystem     *file_system,
 				    const GtkFilePath *path,
 				    GError           **error)
 {
-  gchar *filename;
+  GtkFileSystemUnix *system_unix;
+  const char *filename;
   gboolean result;
+  char *parent;
 
-  filename = filename_from_path (path);
+  system_unix = GTK_FILE_SYSTEM_UNIX (file_system);
+
+  filename = gtk_file_path_get_string (path);
   g_return_val_if_fail (filename != NULL, FALSE);
-  
-  result = mkdir (filename, 0777) != 0;
-  
+  g_return_val_if_fail (g_path_is_absolute (filename), FALSE);
+
+  result = mkdir (filename, 0777) == 0;
+
   if (!result)
     {
       gchar *filename_utf8 = g_filename_to_utf8 (filename, -1, NULL, NULL, NULL);
       g_set_error (error,
 		   GTK_FILE_SYSTEM_ERROR,
 		   GTK_FILE_SYSTEM_ERROR_NONEXISTENT,
-		   "error creating directory '%s': %s",
+		   _("error creating directory '%s': %s"),
 		   filename_utf8 ? filename_utf8 : "???",
 		   g_strerror (errno));
       g_free (filename_utf8);
+      return FALSE;
     }
-  
-  g_free (filename);
-  
-  return result;
+
+  if (filename_is_root (filename))
+    return TRUE; /* hmmm, but with no notification */
+
+  parent = g_path_get_dirname (filename);
+  if (parent)
+    {
+      GtkFileFolderUnix *folder_unix;
+
+      folder_unix = g_hash_table_lookup (system_unix->folder_hash, parent);
+      if (folder_unix)
+	{
+	  GSList *paths;
+
+	  paths = g_slist_append (NULL, (GtkFilePath *) path);
+	  g_signal_emit_by_name (folder_unix, "files-added", paths);
+	  g_slist_free (paths);
+	}
+    }
+
+  return TRUE;
+}
+
+static void
+gtk_file_system_unix_volume_free (GtkFileSystem        *file_system,
+				  GtkFileSystemVolume  *volume)
+{
+  GtkFilePath *path;
+
+  path = (GtkFilePath *) volume;
+  gtk_file_path_free (path);
+}
+
+static GtkFilePath *
+gtk_file_system_unix_volume_get_base_path (GtkFileSystem        *file_system,
+					   GtkFileSystemVolume  *volume)
+{
+  return gtk_file_path_new_dup ("/");
+}
+
+static gboolean
+gtk_file_system_unix_volume_get_is_mounted (GtkFileSystem        *file_system,
+					    GtkFileSystemVolume  *volume)
+{
+  return TRUE;
+}
+
+static gboolean
+gtk_file_system_unix_volume_mount (GtkFileSystem        *file_system,
+				   GtkFileSystemVolume  *volume,
+				   GError              **error)
+{
+  g_set_error (error,
+	       GTK_FILE_SYSTEM_ERROR,
+	       GTK_FILE_SYSTEM_ERROR_FAILED,
+	       _("This file system does not support mounting"));
+  return FALSE;
+}
+
+static gchar *
+gtk_file_system_unix_volume_get_display_name (GtkFileSystem       *file_system,
+					      GtkFileSystemVolume *volume)
+{
+  return g_strdup (_("Filesystem")); /* Same as Nautilus */
+}
+
+static IconType
+get_icon_type (const char *filename,
+	       GError    **error)
+{
+  struct stat statbuf;
+  IconType icon_type;
+
+  /* If stat fails, try to fall back to lstat to catch broken links
+   */
+  if (stat (filename, &statbuf) != 0 &&
+      lstat (filename, &statbuf) != 0)
+    {
+      gchar *filename_utf8 = g_filename_to_utf8 (filename, -1, NULL, NULL, NULL);
+      g_set_error (error,
+		   GTK_FILE_SYSTEM_ERROR,
+		   GTK_FILE_SYSTEM_ERROR_NONEXISTENT,
+		   _("error getting information for '%s': %s"),
+		   filename_utf8 ? filename_utf8 : "???",
+		   g_strerror (errno));
+      g_free (filename_utf8);
+
+      return ICON_NONE;
+    }
+
+  if (S_ISBLK (statbuf.st_mode))
+    icon_type = ICON_BLOCK_DEVICE;
+  else if (S_ISLNK (statbuf.st_mode))
+    icon_type = ICON_BROKEN_SYMBOLIC_LINK; /* See above */
+  else if (S_ISCHR (statbuf.st_mode))
+    icon_type = ICON_CHARACTER_DEVICE;
+  else if (S_ISDIR (statbuf.st_mode))
+    icon_type = ICON_DIRECTORY;
+  else if (S_ISFIFO (statbuf.st_mode))
+    icon_type =  ICON_FIFO;
+  else if (S_ISSOCK (statbuf.st_mode))
+    icon_type = ICON_SOCKET;
+  else
+    {
+      icon_type = ICON_REGULAR;
+
+#if 0
+      if ((types & GTK_FILE_INFO_ICON) && icon_type == GTK_FILE_ICON_REGULAR &&
+	  (statbuf.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)) &&
+	  (strcmp (mime_type, XDG_MIME_TYPE_UNKNOWN) == 0 ||
+	   strcmp (mime_type, "application/x-executable") == 0 ||
+	   strcmp (mime_type, "application/x-shellscript") == 0))
+	gtk_file_info_set_icon_type (info, GTK_FILE_ICON_EXECUTABLE);
+#endif
+    }
+
+  return icon_type;
+}
+
+typedef struct
+{
+  gint size;
+  GdkPixbuf *pixbuf;
+} IconCacheElement;
+
+static void
+icon_cache_element_free (IconCacheElement *element)
+{
+  if (element->pixbuf)
+    g_object_unref (element->pixbuf);
+  g_free (element);
+}
+
+static void
+icon_theme_changed (GtkIconTheme *icon_theme)
+{
+  GHashTable *cache;
+
+  /* Difference from the initial creation is that we don't
+   * reconnect the signal
+   */
+  cache = g_hash_table_new_full (g_str_hash, g_str_equal,
+				 (GDestroyNotify)g_free,
+				 (GDestroyNotify)icon_cache_element_free);
+  g_object_set_data_full (G_OBJECT (icon_theme), "gtk-file-icon-cache",
+			  cache, (GDestroyNotify)g_hash_table_destroy);
+}
+
+static GdkPixbuf *
+get_cached_icon (GtkWidget   *widget,
+		 const gchar *name,
+		 gint         pixel_size)
+{
+  GtkIconTheme *icon_theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (widget));
+  GHashTable *cache = g_object_get_data (G_OBJECT (icon_theme), "gtk-file-icon-cache");
+  IconCacheElement *element;
+
+  if (!cache)
+    {
+      cache = g_hash_table_new_full (g_str_hash, g_str_equal,
+				     (GDestroyNotify)g_free,
+				     (GDestroyNotify)icon_cache_element_free);
+
+      g_object_set_data_full (G_OBJECT (icon_theme), "gtk-file-icon-cache",
+			      cache, (GDestroyNotify)g_hash_table_destroy);
+      g_signal_connect (icon_theme, "changed",
+			G_CALLBACK (icon_theme_changed), NULL);
+    }
+
+  element = g_hash_table_lookup (cache, name);
+  if (!element)
+    {
+      element = g_new0 (IconCacheElement, 1);
+      g_hash_table_insert (cache, g_strdup (name), element);
+    }
+
+  if (element->size != pixel_size)
+    {
+      if (element->pixbuf)
+	g_object_unref (element->pixbuf);
+      element->size = pixel_size;
+      element->pixbuf = gtk_icon_theme_load_icon (icon_theme, name,
+						  pixel_size, 0, NULL);
+    }
+
+  return element->pixbuf ? g_object_ref (element->pixbuf) : NULL;
+}
+
+static GdkPixbuf *
+gtk_file_system_unix_volume_render_icon (GtkFileSystem        *file_system,
+					 GtkFileSystemVolume  *volume,
+					 GtkWidget            *widget,
+					 gint                  pixel_size,
+					 GError              **error)
+{
+  /* FIXME: set the GError if we can't load the icon */
+  return get_cached_icon (widget, "gnome-fs-blockdev", pixel_size);
 }
 
 static gboolean
@@ -312,8 +590,11 @@ gtk_file_system_unix_get_parent (GtkFileSystem     *file_system,
 				 GtkFilePath      **parent,
 				 GError           **error)
 {
-  gchar *filename = filename_from_path (path);
+  const char *filename;
+
+  filename = gtk_file_path_get_string (path);
   g_return_val_if_fail (filename != NULL, FALSE);
+  g_return_val_if_fail (g_path_is_absolute (filename), FALSE);
 
   if (filename_is_root (filename))
     {
@@ -326,8 +607,6 @@ gtk_file_system_unix_get_parent (GtkFileSystem     *file_system,
       g_free (parent_filename);
     }
 
-  g_free (filename);
-
   return TRUE;
 }
 
@@ -337,14 +616,15 @@ gtk_file_system_unix_make_path (GtkFileSystem    *file_system,
 			       const gchar       *display_name,
 			       GError           **error)
 {
-  gchar *base_filename;
+  const char *base_filename;
   gchar *filename;
   gchar *full_filename;
   GError *tmp_error = NULL;
   GtkFilePath *result;
-  
-  base_filename = filename_from_path (base_path);
+
+  base_filename = gtk_file_path_get_string (base_path);
   g_return_val_if_fail (base_filename != NULL, NULL);
+  g_return_val_if_fail (g_path_is_absolute (base_filename), NULL);
 
   filename = g_filename_from_utf8 (display_name, -1, NULL, NULL, &tmp_error);
   if (!filename)
@@ -354,19 +634,17 @@ gtk_file_system_unix_make_path (GtkFileSystem    *file_system,
 		   GTK_FILE_SYSTEM_ERROR_BAD_FILENAME,
 		   "%s",
 		   tmp_error->message);
-      
-      g_error_free (tmp_error);
-      g_free (base_filename);
 
-      return FALSE;
+      g_error_free (tmp_error);
+
+      return NULL;
     }
-    
+
   full_filename = g_build_filename (base_filename, filename, NULL);
   result = filename_to_path (full_filename);
-  g_free (base_filename);
   g_free (filename);
   g_free (full_filename);
-  
+
   return result;
 }
 
@@ -401,7 +679,7 @@ canonicalize_filename (gchar *filename)
 		{
 		  if (*(p + 1) == '\0')
 		    break;
-		  
+
 		  p += 1;
 		}
 	      else if (*(p + 1) == '.' &&
@@ -418,7 +696,7 @@ canonicalize_filename (gchar *filename)
 
 		  if (*(p + 2) == '\0')
 		    break;
-		  
+
 		  p += 2;
 		}
 	      else
@@ -451,13 +729,14 @@ gtk_file_system_unix_parse (GtkFileSystem     *file_system,
 			    gchar            **file_part,
 			    GError           **error)
 {
-  char *base_filename;
+  const char *base_filename;
   gchar *last_slash;
   gboolean result = FALSE;
 
-  base_filename = filename_from_path (base_path);
+  base_filename = gtk_file_path_get_string (base_path);
   g_return_val_if_fail (base_filename != NULL, FALSE);
-  
+  g_return_val_if_fail (g_path_is_absolute (base_filename), FALSE);
+
   last_slash = strrchr (str, G_DIR_SEPARATOR);
   if (!last_slash)
     {
@@ -497,7 +776,7 @@ gtk_file_system_unix_parse (GtkFileSystem     *file_system,
 	    }
 
 	  canonicalize_filename (folder_path);
-	  
+
 	  *folder = filename_to_path (folder_path);
 	  *file_part = g_strdup (last_slash + 1);
 
@@ -507,8 +786,6 @@ gtk_file_system_unix_parse (GtkFileSystem     *file_system,
 	}
     }
 
-  g_free (base_filename);
-  
   return result;
 }
 
@@ -544,17 +821,299 @@ gtk_file_system_unix_filename_to_path (GtkFileSystem *file_system,
   return gtk_file_path_new_dup (filename);
 }
 
+static GdkPixbuf *
+gtk_file_system_unix_render_icon (GtkFileSystem     *file_system,
+				  const GtkFilePath *path,
+				  GtkWidget         *widget,
+				  gint               pixel_size,
+				  GError           **error)
+{
+  const char *filename;
+  IconType icon_type;
+  const char *mime_type;
+
+  filename = gtk_file_path_get_string (path);
+  icon_type = get_icon_type (filename, error);
+
+  /* FIXME: this function should not return NULL without setting the GError; we
+   * should perhaps provide a "never fails" generic stock icon for when all else
+   * fails.
+   */
+
+  if (icon_type == ICON_NONE)
+    return NULL;
+
+  if (icon_type != ICON_REGULAR)
+    {
+      const char *name;
+
+      switch (icon_type)
+	{
+	case ICON_BLOCK_DEVICE:
+          name = "gnome-fs-blockdev";
+	  break;
+	case ICON_BROKEN_SYMBOLIC_LINK:
+	  name = "gnome-fs-symlink";
+	  break;
+	case ICON_CHARACTER_DEVICE:
+	  name = "gnome-fs-chardev";
+	  break;
+	case ICON_DIRECTORY:
+	  name = "gnome-fs-directory";
+	  break;
+	case ICON_EXECUTABLE:
+	  name ="gnome-fs-executable";
+	  break;
+	case ICON_FIFO:
+	  name = "gnome-fs-fifo";
+	  break;
+	case ICON_SOCKET:
+	  name = "gnome-fs-socket";
+	  break;
+	default:
+	  g_assert_not_reached ();
+	  return NULL;
+	}
+
+      return get_cached_icon (widget, name, pixel_size);
+    }
+
+  mime_type = xdg_mime_get_mime_type_for_file (filename);
+  if (mime_type)
+    {
+      const char *separator;
+      GString *icon_name;
+      GdkPixbuf *pixbuf;
+
+      separator = strchr (mime_type, '/');
+      if (!separator)
+	return NULL;
+
+      icon_name = g_string_new ("gnome-mime-");
+      g_string_append_len (icon_name, mime_type, separator - mime_type);
+      g_string_append_c (icon_name, '-');
+      g_string_append (icon_name, separator + 1);
+      pixbuf = get_cached_icon (widget, icon_name->str, pixel_size);
+      g_string_free (icon_name, TRUE);
+      if (pixbuf)
+	return pixbuf;
+
+      icon_name = g_string_new ("gnome-mime-");
+      g_string_append_len (icon_name, mime_type, separator - mime_type);
+      pixbuf = get_cached_icon (widget, icon_name->str, pixel_size);
+      g_string_free (icon_name, TRUE);
+      if (pixbuf)
+	return pixbuf;
+    }
+
+  return get_cached_icon (widget, "gnome-fs-regular", pixel_size);
+}
+
+static void
+bookmark_list_free (GSList *list)
+{
+  GSList *l;
+
+  for (l = list; l; l = l->next)
+    g_free (l->data);
+
+  g_slist_free (list);
+}
+
+/* Returns whether a URI is a local file:// */
+static gboolean
+is_local_uri (const char *uri)
+{
+  char *filename;
+  char *hostname;
+  gboolean result;
+
+  /* This is rather crude, but hey */
+  filename = g_filename_from_uri (uri, &hostname, NULL);
+
+  result = (filename && !hostname);
+
+  g_free (filename);
+  g_free (hostname);
+
+  return result;
+}
+
+static char *
+bookmark_get_filename (gboolean tmp_file)
+{
+  char *filename;
+
+  filename = g_build_filename (g_get_home_dir (),
+			       tmp_file ? BOOKMARKS_TMP_FILENAME : BOOKMARKS_FILENAME,
+			       NULL);
+  g_assert (filename != NULL);
+  return filename;
+}
+
+static gboolean
+bookmark_list_read (GSList **bookmarks, GError **error)
+{
+  gchar *filename;
+  gchar *contents;
+  gboolean result = FALSE;
+
+  filename = bookmark_get_filename (FALSE);
+  *bookmarks = NULL;
+
+  if (g_file_get_contents (filename, &contents, NULL, error))
+    {
+      gchar **lines = g_strsplit (contents, "\n", -1);
+      int i;
+      GHashTable *table;
+
+      table = g_hash_table_new (g_str_hash, g_str_equal);
+
+      for (i = 0; lines[i]; i++)
+	{
+	  if (lines[i][0] && !g_hash_table_lookup (table, lines[i]))
+	    {
+	      *bookmarks = g_slist_prepend (*bookmarks, g_strdup (lines[i]));
+	      g_hash_table_insert (table, lines[i], lines[i]);
+	    }
+	}
+
+      g_free (contents);
+      g_hash_table_destroy (table);
+      g_strfreev (lines);
+
+      *bookmarks = g_slist_reverse (*bookmarks);
+      result = TRUE;
+    }
+
+  g_free (filename);
+
+  return result;
+}
+
+static gboolean
+bookmark_list_write (GSList *bookmarks, GError **error)
+{
+  char *tmp_filename;
+  char *filename;
+  gboolean result = TRUE;
+  FILE *file;
+  int fd;
+  int saved_errno;
+
+  /* First, write a temporary file */
+
+  tmp_filename = bookmark_get_filename (TRUE);
+  filename = bookmark_get_filename (FALSE);
+
+  fd = g_mkstemp (tmp_filename);
+  if (fd == -1)
+    {
+      saved_errno = errno;
+      goto io_error;
+    }
+
+  if ((file = fdopen (fd, "w")) != NULL)
+    {
+      GSList *l;
+
+      for (l = bookmarks; l; l = l->next)
+	if (fputs (l->data, file) == EOF
+	    || fputs ("\n", file) == EOF)
+	  {
+	    saved_errno = errno;
+	    goto io_error;
+	  }
+
+      if (fclose (file) == EOF)
+	{
+	  saved_errno = errno;
+	  goto io_error;
+	}
+
+      if (rename (tmp_filename, filename) == -1)
+	{
+	  saved_errno = errno;
+	  goto io_error;
+	}
+
+      result = TRUE;
+      goto out;
+    }
+  else
+    {
+      saved_errno = errno;
+
+      /* fdopen() failed, so we can't do much error checking here anyway */
+      close (fd);
+    }
+
+ io_error:
+
+  g_set_error (error,
+	       GTK_FILE_SYSTEM_ERROR,
+	       GTK_FILE_SYSTEM_ERROR_FAILED,
+	       _("Bookmark saving failed (%s)"),
+	       g_strerror (saved_errno));
+  result = FALSE;
+
+  if (fd != -1)
+    unlink (tmp_filename); /* again, not much error checking we can do here */
+
+ out:
+
+  g_free (filename);
+  g_free (tmp_filename);
+
+  return result;
+}
+
 static gboolean
 gtk_file_system_unix_add_bookmark (GtkFileSystem     *file_system,
 				   const GtkFilePath *path,
 				   GError           **error)
 {
-  /* FIXME: Implement as a really simple ~/.gtk-bookmarks */
-  g_set_error (error,
-	       GTK_FILE_SYSTEM_ERROR,
-	       GTK_FILE_SYSTEM_ERROR_FAILED,
-	       "This file system does not support bookmarks");
-  return FALSE;
+  GSList *bookmarks;
+  GSList *l;
+  char *uri;
+  gboolean result;
+  GError *err;
+
+  err = NULL;
+  if (!bookmark_list_read (&bookmarks, &err) && err->code != G_FILE_ERROR_NOENT)
+    {
+      g_propagate_error (error, err);
+      g_error_free (err);
+      return FALSE;
+    }
+
+  result = FALSE;
+
+  uri = gtk_file_system_unix_path_to_uri (file_system, path);
+
+  for (l = bookmarks; l; l = l->next)
+    {
+      const char *bookmark;
+
+      bookmark = l->data;
+      if (strcmp (bookmark, uri) == 0)
+	break;
+    }
+
+  if (!l)
+    {
+      bookmarks = g_slist_append (bookmarks, g_strdup (uri));
+      if (bookmark_list_write (bookmarks, error))
+	{
+	  result = TRUE;
+	  g_signal_emit_by_name (file_system, "bookmarks-changed", 0);
+	}
+    }
+
+  g_free (uri);
+  bookmark_list_free (bookmarks);
+
+  return result;
 }
 
 static gboolean
@@ -562,19 +1121,74 @@ gtk_file_system_unix_remove_bookmark (GtkFileSystem     *file_system,
 				      const GtkFilePath *path,
 				      GError           **error)
 {
-  /* FIXME: Implement as a really simple ~/.gtk-bookmarks */
-  g_set_error (error,
-	       GTK_FILE_SYSTEM_ERROR,
-	       GTK_FILE_SYSTEM_ERROR_FAILED,
-	       "This file system does not support bookmarks");
-  return FALSE;
+  GSList *bookmarks;
+  char *uri;
+  GSList *l;
+  gboolean result;
+
+  if (!bookmark_list_read (&bookmarks, error))
+    return FALSE;
+
+  result = FALSE;
+
+  uri = gtk_file_system_path_to_uri (file_system, path);
+
+  for (l = bookmarks; l; l = l->next)
+    {
+      const char *bookmark;
+
+      bookmark = l->data;
+      if (strcmp (bookmark, uri) == 0)
+	break;
+    }
+
+  if (l)
+    {
+      g_free (l->data);
+      bookmarks = g_slist_remove_link (bookmarks, l);
+      g_slist_free_1 (l);
+
+      if (bookmark_list_write (bookmarks, error))
+	result = TRUE;
+    }
+  else
+    result = TRUE;
+
+  g_free (uri);
+  bookmark_list_free (bookmarks);
+
+  if (result)
+    g_signal_emit_by_name (file_system, "bookmarks-changed", 0);
+
+  return result;
 }
 
 static GSList *
 gtk_file_system_unix_list_bookmarks (GtkFileSystem *file_system)
 {
-  /* FIXME: Implement as a really simple ~/.gtk-bookmarks */
-  return NULL;
+  GSList *bookmarks;
+  GSList *result;
+  GSList *l;
+
+  if (!bookmark_list_read (&bookmarks, NULL))
+    return NULL;
+
+  result = NULL;
+
+  for (l = bookmarks; l; l = l->next)
+    {
+      const char *name;
+
+      name = l->data;
+
+      if (is_local_uri (name))
+	result = g_slist_prepend (result, gtk_file_system_unix_uri_to_path (file_system, name));
+    }
+
+  bookmark_list_free (bookmarks);
+
+  result = g_slist_reverse (result);
+  return result;
 }
 
 /*
@@ -599,7 +1213,7 @@ gtk_file_folder_unix_get_type (void)
 	0,		/* n_preallocs */
 	(GInstanceInitFunc) gtk_file_folder_unix_init,
       };
-      
+
       static const GInterfaceInfo file_folder_info =
       {
 	(GInterfaceInitFunc) gtk_file_folder_unix_iface_init, /* interface_init */
@@ -645,8 +1259,10 @@ gtk_file_folder_unix_finalize (GObject *object)
 {
   GtkFileFolderUnix *folder_unix = GTK_FILE_FOLDER_UNIX (object);
 
+  g_hash_table_remove (folder_unix->system_unix->folder_hash, folder_unix->filename);
+
   g_free (folder_unix->filename);
-  
+
   folder_parent_class->finalize (object);
 }
 
@@ -658,10 +1274,11 @@ gtk_file_folder_unix_get_info (GtkFileFolder  *folder,
   GtkFileFolderUnix *folder_unix = GTK_FILE_FOLDER_UNIX (folder);
   GtkFileInfo *info;
   gchar *dirname;
-  gchar *filename;
-  
-  filename = filename_from_path (path);
+  const char *filename;
+
+  filename = gtk_file_path_get_string (path);
   g_return_val_if_fail (filename != NULL, NULL);
+  g_return_val_if_fail (g_path_is_absolute (filename), NULL);
 
   dirname = g_path_get_dirname (filename);
   g_return_val_if_fail (strcmp (dirname, folder_unix->filename) == 0, NULL);
@@ -669,8 +1286,6 @@ gtk_file_folder_unix_get_info (GtkFileFolder  *folder,
 
   info = filename_get_info (filename, folder_unix->types, error);
 
-  g_free (filename);
-
   return info;
 }
 
@@ -693,7 +1308,7 @@ gtk_file_folder_unix_list_children (GtkFileFolder  *folder,
 		   GTK_FILE_SYSTEM_ERROR_NONEXISTENT,
 		   "%s",
 		   tmp_error->message);
-      
+
       g_error_free (tmp_error);
 
       return FALSE;
@@ -715,7 +1330,7 @@ gtk_file_folder_unix_list_children (GtkFileFolder  *folder,
   g_dir_close (dir);
 
   *children = g_slist_reverse (*children);
-  
+
   return TRUE;
 }
 
@@ -725,9 +1340,8 @@ filename_get_info (const gchar     *filename,
 		   GError         **error)
 {
   GtkFileInfo *info;
-  GtkFileIconType icon_type = GTK_FILE_ICON_REGULAR;
   struct stat statbuf;
-  
+
   /* If stat fails, try to fall back to lstat to catch broken links
    */
   if (stat (filename, &statbuf) != 0 &&
@@ -737,11 +1351,11 @@ filename_get_info (const gchar     *filename,
       g_set_error (error,
 		   GTK_FILE_SYSTEM_ERROR,
 		   GTK_FILE_SYSTEM_ERROR_NONEXISTENT,
-		   "error getting information for '%s': %s",
+		   _("error getting information for '%s': %s"),
 		   filename_utf8 ? filename_utf8 : "???",
 		   g_strerror (errno));
       g_free (filename_utf8);
-      
+
       return NULL;
     }
 
@@ -751,25 +1365,25 @@ filename_get_info (const gchar     *filename,
     {
       if (types & GTK_FILE_INFO_DISPLAY_NAME)
 	gtk_file_info_set_display_name (info, "/");
-      
+
       if (types & GTK_FILE_INFO_IS_HIDDEN)
 	gtk_file_info_set_is_hidden (info, FALSE);
     }
   else
     {
       gchar *basename = g_path_get_basename (filename);
-  
+
       if (types & GTK_FILE_INFO_DISPLAY_NAME)
 	{
 	  gchar *display_name = g_filename_to_utf8 (basename, -1, NULL, NULL, NULL);
 	  if (!display_name)
 	    display_name = g_strescape (basename, NULL);
-	  
+
 	  gtk_file_info_set_display_name (info, display_name);
-	  
+
 	  g_free (display_name);
 	}
-      
+
       if (types & GTK_FILE_INFO_IS_HIDDEN)
 	{
 	  gtk_file_info_set_is_hidden (info, basename[0] == '.');
@@ -783,36 +1397,10 @@ filename_get_info (const gchar     *filename,
       gtk_file_info_set_is_folder (info, S_ISDIR (statbuf.st_mode));
    }
 
-  if (types & GTK_FILE_INFO_ICON)
-    {
-      if (S_ISBLK (statbuf.st_mode))
-	icon_type = GTK_FILE_ICON_BLOCK_DEVICE;
-      else if (S_ISLNK (statbuf.st_mode))
-	icon_type = GTK_FILE_ICON_BROKEN_SYMBOLIC_LINK;
-      else if (S_ISCHR (statbuf.st_mode))
-	icon_type = GTK_FILE_ICON_CHARACTER_DEVICE;
-      else if (S_ISDIR (statbuf.st_mode))
-	icon_type = GTK_FILE_ICON_DIRECTORY;
-      else if (S_ISFIFO (statbuf.st_mode))
-	icon_type =  GTK_FILE_ICON_FIFO;
-      else if (S_ISSOCK (statbuf.st_mode))
-	icon_type = GTK_FILE_ICON_SOCKET;
-
-      gtk_file_info_set_icon_type (info, icon_type);
-    }
-
-  if ((types & GTK_FILE_INFO_MIME_TYPE) ||
-      ((types & GTK_FILE_INFO_ICON) && icon_type == GTK_FILE_ICON_REGULAR))
+  if (types & GTK_FILE_INFO_MIME_TYPE)
     {
       const char *mime_type = xdg_mime_get_mime_type_for_file (filename);
       gtk_file_info_set_mime_type (info, mime_type);
-
-      if ((types & GTK_FILE_INFO_ICON) && icon_type == GTK_FILE_ICON_REGULAR &&
-	  (statbuf.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)) &&
-	  (strcmp (mime_type, XDG_MIME_TYPE_UNKNOWN) == 0 ||
-	   strcmp (mime_type, "application/x-executable") == 0 ||
-	   strcmp (mime_type, "application/x-shellscript") == 0))
-	gtk_file_info_set_icon_type (info, GTK_FILE_ICON_EXECUTABLE);
     }
 
   if (types & GTK_FILE_INFO_MODIFICATION_TIME)
@@ -824,14 +1412,8 @@ filename_get_info (const gchar     *filename,
     {
       gtk_file_info_set_size (info, (gint64)statbuf.st_size);
     }
-  
-  return info;
-}
 
-static gchar *
-filename_from_path (const GtkFilePath *path)
-{
-  return g_strdup (gtk_file_path_get_string (path));
+  return info;
 }
 
 static GtkFilePath *
@@ -844,8 +1426,8 @@ static gboolean
 filename_is_root (const char *filename)
 {
   const gchar *after_root;
-  
+
   after_root = g_path_skip_root (filename);
-  
-  return (*after_root == '\0');
+
+  return (after_root != NULL && *after_root == '\0');
 }
