@@ -1042,6 +1042,28 @@ gtk_real_menu_shell_move_current (GtkMenuShell      *menu_shell,
   if (menu_shell->parent_menu_shell)
     parent_menu_shell = GTK_MENU_SHELL (menu_shell->parent_menu_shell);
   
+  if (gtk_widget_get_direction (GTK_WIDGET (menu_shell)) == GTK_TEXT_DIR_RTL)
+    {
+      switch (direction) 
+	{
+	case GTK_MENU_DIR_PARENT:
+	  direction = GTK_MENU_DIR_CHILD;
+	  break;
+	case GTK_MENU_DIR_CHILD:
+	  direction = GTK_MENU_DIR_PARENT;
+	  break;
+	case GTK_MENU_DIR_PREV:
+	  if (GTK_MENU_SHELL_GET_CLASS (menu_shell)->submenu_placement == GTK_TOP_BOTTOM)
+	    direction = GTK_MENU_DIR_NEXT;
+	  break;
+	case GTK_MENU_DIR_NEXT:
+	  if (GTK_MENU_SHELL_GET_CLASS (menu_shell)->submenu_placement == GTK_TOP_BOTTOM)
+	    direction = GTK_MENU_DIR_PREV;
+	  break;
+	default: ;
+	}
+    }
+  
   switch (direction)
     {
     case GTK_MENU_DIR_PARENT:
@@ -1182,3 +1204,19 @@ _gtk_menu_shell_get_popup_delay (GtkMenuShell *menu_shell)
       return popup_delay;
     }
 }
+
+/**
+ * gtk_menu_shell_cancel:
+ * @menu_shell: a #GtkMenuShell
+ * 
+ * Cancels the selection within the menu shell.  
+ * 
+ * Since: 2.4
+ */
+void
+gtk_menu_shell_cancel (GtkMenuShell *menu_shell)
+{
+  g_return_if_fail (GTK_IS_MENU_SHELL (menu_shell));
+
+  g_signal_emit (menu_shell, menu_shell_signals[CANCEL], 0);
+}
