@@ -77,7 +77,6 @@
  */
 
 #define GTK_TEXT_USE_INTERNAL_UNSUPPORTED_API
-#include "gtksignal.h"
 #include "gtkmarshalers.h"
 #include "gtktextlayout.h"
 #include "gtktextbtree.h"
@@ -102,6 +101,10 @@ static void gtk_text_layout_invalidate_cursor_line (GtkTextLayout     *layout);
 static void gtk_text_layout_real_free_line_data    (GtkTextLayout     *layout,
 						    GtkTextLine       *line,
 						    GtkTextLineData   *line_data);
+static void gtk_text_layout_emit_changed           (GtkTextLayout     *layout,
+						    gint               y,
+						    gint               old_height,
+						    gint               new_height);
 
 static void gtk_text_layout_invalidate_all (GtkTextLayout *layout);
 
@@ -119,6 +122,8 @@ enum {
   LAST_ARG
 };
 
+#define PIXEL_BOUND(d) (((d) + PANGO_SCALE - 1) / PANGO_SCALE)
+
 static void gtk_text_layout_init       (GtkTextLayout      *text_layout);
 static void gtk_text_layout_class_init (GtkTextLayoutClass *klass);
 static void gtk_text_layout_finalize   (GObject            *object);
@@ -149,10 +154,8 @@ gtk_text_layout_get_type (void)
         (GInstanceInitFunc) gtk_text_layout_init
       };
 
-      our_type = g_type_register_static (G_TYPE_OBJECT,
-                                         "GtkTextLayout",
-                                         &our_info,
-                                         0);
+      our_type = g_type_register_static (G_TYPE_OBJECT, "GtkTextLayout",
+                                         &our_info, 0);
     }
 
   return our_type;
@@ -178,7 +181,7 @@ gtk_text_layout_class_init (GtkTextLayoutClass *klass)
                   G_STRUCT_OFFSET (GtkTextLayoutClass, invalidated),
                   NULL, NULL,
                   _gtk_marshal_VOID__VOID,
-                  GTK_TYPE_NONE,
+                  G_TYPE_NONE,
                   0);
 
   signals[CHANGED] =
@@ -188,11 +191,11 @@ gtk_text_layout_class_init (GtkTextLayoutClass *klass)
                   G_STRUCT_OFFSET (GtkTextLayoutClass, changed),
                   NULL, NULL,
                   _gtk_marshal_VOID__INT_INT_INT,
-                  GTK_TYPE_NONE,
+                  G_TYPE_NONE,
                   3,
-                  GTK_TYPE_INT,
-                  GTK_TYPE_INT,
-                  GTK_TYPE_INT);
+                  G_TYPE_INT,
+                  G_TYPE_INT,
+                  G_TYPE_INT);
 
   signals[ALLOCATE_CHILD] =
     g_signal_new ("allocate_child",
@@ -201,11 +204,11 @@ gtk_text_layout_class_init (GtkTextLayoutClass *klass)
                   G_STRUCT_OFFSET (GtkTextLayoutClass, allocate_child),
                   NULL, NULL,
                   _gtk_marshal_VOID__OBJECT_INT_INT,
-                  GTK_TYPE_NONE,
+                  G_TYPE_NONE,
                   3,
                   GTK_TYPE_OBJECT,
-                  GTK_TYPE_INT,
-                  GTK_TYPE_INT);
+                  G_TYPE_INT,
+                  G_TYPE_INT);
 }
 
 static void
@@ -217,7 +220,7 @@ gtk_text_layout_init (GtkTextLayout *text_layout)
 GtkTextLayout*
 gtk_text_layout_new (void)
 {
-  return GTK_TEXT_LAYOUT (g_object_new (gtk_text_layout_get_type (), NULL));
+  return g_object_new (GTK_TYPE_TEXT_LAYOUT, NULL);
 }
 
 static void
@@ -245,15 +248,22 @@ gtk_text_layout_finalize (GObject *object)
 
   if (layout->ltr_context)
     {
-      g_object_unref (G_OBJECT (layout->ltr_context));
+      g_object_unref (layout->ltr_context);
       layout->ltr_context = NULL;
     }
   if (layout->rtl_context)
     {
-      g_object_unref (G_OBJECT (layout->rtl_context));
+      g_object_unref (layout->rtl_context);
       layout->rtl_context = NULL;
     }
   
+  if (layout->one_display_cache) 
+    {
+      GtkTextLineDisplay *tmp_display = layout->one_display_cache;
+      layout->one_display_cache = NULL;
+      gtk_text_layout_free_line_display (layout, tmp_display);
+    }
+
   (* G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
@@ -274,7 +284,7 @@ gtk_text_layout_set_buffer (GtkTextLayout *layout,
       _gtk_text_btree_remove_view (_gtk_text_buffer_get_btree (layout->buffer),
                                   layout);
 
-      g_object_unref (G_OBJECT (layout->buffer));
+      g_object_unref (layout->buffer);
       layout->buffer = NULL;
     }
 
@@ -282,7 +292,7 @@ gtk_text_layout_set_buffer (GtkTextLayout *layout,
     {
       layout->buffer = buffer;
 
-      g_object_ref (G_OBJECT (buffer));
+      g_object_ref (buffer);
 
       _gtk_text_btree_add_view (_gtk_text_buffer_get_btree (buffer), layout);
     }
@@ -325,16 +335,16 @@ gtk_text_layout_set_contexts (GtkTextLayout *layout,
   g_return_if_fail (GTK_IS_TEXT_LAYOUT (layout));
 
   if (layout->ltr_context)
-    g_object_unref (G_OBJECT (ltr_context));
+    g_object_unref (ltr_context);
 
   layout->ltr_context = ltr_context;
-  g_object_ref (G_OBJECT (ltr_context));
+  g_object_ref (ltr_context);
 
   if (layout->rtl_context)
-    g_object_unref (G_OBJECT (rtl_context));
+    g_object_unref (rtl_context);
 
   layout->rtl_context = rtl_context;
-  g_object_ref (G_OBJECT (rtl_context));
+  g_object_ref (rtl_context);
 
   DV (g_print ("invalidating all due to new pango contexts (%s)\n", G_STRLOC));
   gtk_text_layout_invalidate_all (layout);
@@ -426,7 +436,7 @@ gtk_text_layout_set_cursor_visible (GtkTextLayout *layout,
                                         gtk_text_buffer_get_mark (layout->buffer, "insert"));
 
       gtk_text_layout_get_line_yrange (layout, &iter, &y, &height);
-      gtk_text_layout_changed (layout, y, height, height);
+      gtk_text_layout_emit_changed (layout, y, height, height);
 
       gtk_text_layout_invalidate_cache (layout, _gtk_text_iter_get_text_line (&iter));
     }
@@ -511,7 +521,16 @@ gtk_text_layout_get_size (GtkTextLayout *layout,
 static void
 gtk_text_layout_invalidated (GtkTextLayout *layout)
 {
-  g_signal_emit (G_OBJECT (layout), signals[INVALIDATED], 0);
+  g_signal_emit (layout, signals[INVALIDATED], 0);
+}
+
+static void
+gtk_text_layout_emit_changed (GtkTextLayout *layout,
+			      gint           y,
+			      gint           old_height,
+			      gint           new_height)
+{
+  g_signal_emit (layout, signals[CHANGED], 0, y, old_height, new_height);
 }
 
 void
@@ -520,8 +539,21 @@ gtk_text_layout_changed (GtkTextLayout *layout,
                          gint           old_height,
                          gint           new_height)
 {
-  g_signal_emit (G_OBJECT (layout), signals[CHANGED], 0,
-                 y, old_height, new_height);
+  /* Check if the range intersects our cached line display,
+   * and invalidate the cached line if so.
+   */
+  if (layout->one_display_cache)
+    {
+      GtkTextLine *line = layout->one_display_cache->line;
+      gint cache_y = _gtk_text_btree_find_line_top (_gtk_text_buffer_get_btree (layout->buffer),
+						    line, layout);
+      gint cache_height = layout->one_display_cache->height;
+
+      if (cache_y + cache_height > y && cache_y < y + old_height)
+	gtk_text_layout_invalidate_cache (layout, line);
+    }
+  
+  gtk_text_layout_emit_changed (layout, y, old_height, new_height);
 }
 
 void
@@ -775,12 +807,12 @@ update_layout_size (GtkTextLayout *layout)
  * @layout: a #GtkTextLayout
  * @anchor: iter pointing into a line that will be used as the
  *          coordinate origin
- * @y0: offset from the top of the line pointed to by @anchor at
- *      which to begin validation. (The offset here is in pixels
- *      after validation.)
- * @y1: offset from the top of the line pointed to by @anchor at
- *      which to end validation. (The offset here is in pixels
- *      after validation.)
+ * @y0_: offset from the top of the line pointed to by @anchor at
+ *       which to begin validation. (The offset here is in pixels
+ *       after validation.)
+ * @y1_: offset from the top of the line pointed to by @anchor at
+ *       which to end validation. (The offset here is in pixels
+ *       after validation.)
  *
  * Ensure that a region of a #GtkTextLayout is valid. The ::changed
  * signal will be emitted if any lines are validated.
@@ -809,6 +841,7 @@ gtk_text_layout_validate_yrange (GtkTextLayout *layout,
   /* Validate backwards from the anchor line to y0
    */
   line = _gtk_text_iter_get_text_line (anchor);
+  line = _gtk_text_line_previous (line);
   seen = 0;
   while (line && seen < -y0)
     {
@@ -824,11 +857,11 @@ gtk_text_layout_validate_yrange (GtkTextLayout *layout,
           delta_height += line_data->height - old_height;
           
           first_line = line;
-          first_line_y = -seen;
+          first_line_y = -seen - line_data->height;
           if (!last_line)
             {
               last_line = line;
-              last_line_y = -seen + line_data->height;
+              last_line_y = -seen;
             }
         }
 
@@ -877,10 +910,10 @@ gtk_text_layout_validate_yrange (GtkTextLayout *layout,
       line_top = _gtk_text_btree_find_line_top (_gtk_text_buffer_get_btree (layout->buffer),
                                                 first_line, layout);
 
-      gtk_text_layout_changed (layout,
-                               line_top,
-                               last_line_y - first_line_y - delta_height,
-                               last_line_y - first_line_y);
+      gtk_text_layout_emit_changed (layout,
+				    line_top,
+				    last_line_y - first_line_y - delta_height,
+				    last_line_y - first_line_y);
     }
 }
 
@@ -909,7 +942,7 @@ gtk_text_layout_validate (GtkTextLayout *layout,
       max_pixels -= new_height;
 
       update_layout_size (layout);
-      gtk_text_layout_changed (layout, y, old_height, new_height);
+      gtk_text_layout_emit_changed (layout, y, old_height, new_height);
     }
 }
 
@@ -1032,25 +1065,16 @@ totally_invisible_line (GtkTextLayout *layout,
   GtkTextLineSegment *seg;
   int bytes = 0;
 
-  /* If we have a cached style, then we know it does actually apply
-   * and we can just see if it is invisible.
+  /* Check if the first char is visible, if so we are partially visible.  
+   * Note that we have to check this since we don't know the current 
+   * invisible/noninvisible toggle state; this function can use the whole btree 
+   * to get it right.
    */
-  if (layout->one_style_cache &&
-      !layout->one_style_cache->invisible)
+  _gtk_text_btree_get_iter_at_line (_gtk_text_buffer_get_btree (layout->buffer),
+				    iter, line, 0);
+  
+  if (!_gtk_text_btree_char_is_invisible (iter))
     return FALSE;
-  /* Without the cache, we check if the first char is visible, if so
-   * we are partially visible.  Note that we have to check this since
-   * we don't know the current invisible/noninvisible toggle state; this
-   * function can use the whole btree to get it right.
-   */
-  else
-    {
-      _gtk_text_btree_get_iter_at_line (_gtk_text_buffer_get_btree (layout->buffer),
-                                       iter, line, 0);
-
-      if (!_gtk_text_btree_char_is_invisible (iter))
-        return FALSE;
-    }
 
   bytes = 0;
   seg = line->segments;
@@ -1161,6 +1185,12 @@ set_para_values (GtkTextLayout      *layout,
       pango_layout_set_wrap (display->layout, PANGO_WRAP_WORD);
       break;
 
+    case GTK_WRAP_WORD_CHAR:
+      layout_width = layout->screen_width - display->left_margin - display->right_margin;
+      pango_layout_set_width (display->layout, layout_width * PANGO_SCALE);
+      pango_layout_set_wrap (display->layout, PANGO_WRAP_WORD_CHAR);
+      break;
+
     case GTK_WRAP_NONE:
       break;
     }
@@ -1182,9 +1212,9 @@ gtk_text_attr_appearance_destroy (PangoAttribute *attr)
   GtkTextAppearance *appearance = &((GtkTextAttrAppearance *)attr)->appearance;
 
   if (appearance->bg_stipple)
-    gdk_drawable_unref (appearance->bg_stipple);
+    g_object_unref (appearance->bg_stipple);
   if (appearance->fg_stipple)
-    gdk_drawable_unref (appearance->fg_stipple);
+    g_object_unref (appearance->fg_stipple);
 
   g_free (attr);
 }
@@ -1238,9 +1268,9 @@ gtk_text_attr_appearance_new (const GtkTextAppearance *appearance)
   result->appearance = *appearance;
 
   if (appearance->bg_stipple)
-    gdk_drawable_ref (appearance->bg_stipple);
+    g_object_ref (appearance->bg_stipple);
   if (appearance->fg_stipple)
-    gdk_drawable_ref (appearance->fg_stipple);
+    g_object_ref (appearance->fg_stipple);
 
   return (PangoAttribute *)result;
 }
@@ -1533,7 +1563,7 @@ allocate_child_widgets (GtkTextLayout      *text_layout,
                                                  NULL,
                                                  &extents);
               
-              g_signal_emit (G_OBJECT (text_layout),
+              g_signal_emit (text_layout,
                              signals[ALLOCATE_CHILD],
                              0,
                              shaped_object,
@@ -1584,6 +1614,9 @@ add_preedit_attrs (GtkTextLayout     *layout,
       if (end == G_MAXINT)
 	end = layout->preedit_len;
       
+      if (end == start)
+	continue;
+
       pango_attr_iterator_get_font (iter, font_desc, &language, &extra_attrs);
       
       tmp_list = extra_attrs;
@@ -1941,7 +1974,7 @@ gtk_text_layout_get_line_display (GtkTextLayout *layout,
 
   pango_layout_get_extents (display->layout, NULL, &extents);
 
-  display->width = PANGO_PIXELS (extents.width) + display->left_margin + display->right_margin;
+  display->width = PIXEL_BOUND (extents.width) + display->left_margin + display->right_margin;
   display->height += PANGO_PIXELS (extents.height);
   
   /* Free this if we aren't in a loop */
@@ -1966,7 +1999,7 @@ gtk_text_layout_free_line_display (GtkTextLayout      *layout,
   if (display != layout->one_display_cache)
     {
       if (display->layout)
-        g_object_unref (G_OBJECT (display->layout));
+        g_object_unref (display->layout);
 
       if (display->cursors)
         {
@@ -1993,8 +2026,11 @@ line_display_iter_to_index (GtkTextLayout      *layout,
 
   index = gtk_text_iter_get_visible_line_index (iter);
   
-  if (index >= display->insert_index)
-    index += layout->preedit_len;
+  if (layout->preedit_len > 0 && display->insert_index >= 0)
+    {
+      if (index >= display->insert_index)
+	index += layout->preedit_len;
+    }
 
   return index;
 }
@@ -2009,12 +2045,15 @@ line_display_index_to_iter (GtkTextLayout      *layout,
   g_return_if_fail (!_gtk_text_line_is_last (display->line,
                                              _gtk_text_buffer_get_btree (layout->buffer)));
   
-  if (index >= display->insert_index + layout->preedit_len)
-    index -= layout->preedit_len;
-  else if (index > display->insert_index)
+  if (layout->preedit_len > 0 && display->insert_index >= 0)
     {
-      index = display->insert_index;
-      trailing = 0;
+      if (index >= display->insert_index + layout->preedit_len)
+	index -= layout->preedit_len;
+      else if (index > display->insert_index)
+	{
+	  index = display->insert_index;
+	  trailing = 0;
+	}
     }
 
   _gtk_text_btree_get_iter_at_line (_gtk_text_buffer_get_btree (layout->buffer),
@@ -2132,13 +2171,13 @@ gtk_text_layout_get_iter_at_pixel (GtkTextLayout *layout,
 }
 
 /**
- * gtk_text_layout_get_cursor_locations
+ * gtk_text_layout_get_cursor_locations:
  * @layout: a #GtkTextLayout
  * @iter: a #GtkTextIter
  * @strong_pos: location to store the strong cursor position (may be %NULL)
  * @weak_pos: location to store the weak cursor position (may be %NULL)
  *
- * Given an iterator within a text laout, determine the positions that of the
+ * Given an iterator within a text layout, determine the positions of the
  * strong and weak cursors if the insertion point is at that
  * iterator. The position of each cursor is stored as a zero-width
  * rectangle. The strong cursor location is the location where
@@ -2157,6 +2196,7 @@ gtk_text_layout_get_cursor_locations (GtkTextLayout  *layout,
   GtkTextLineDisplay *display;
   gint line_top;
   gint index;
+  GtkTextIter insert_iter;
 
   PangoRectangle pango_strong_pos;
   PangoRectangle pango_weak_pos;
@@ -2171,6 +2211,13 @@ gtk_text_layout_get_cursor_locations (GtkTextLayout  *layout,
   line_top = _gtk_text_btree_find_line_top (_gtk_text_buffer_get_btree (layout->buffer),
                                            line, layout);
   
+  gtk_text_buffer_get_iter_at_mark (layout->buffer, &insert_iter,
+                                    gtk_text_buffer_get_mark (layout->buffer,
+                                                              "insert"));
+
+  if (gtk_text_iter_equal (iter, &insert_iter))
+    index += layout->preedit_cursor - layout->preedit_len;
+  
   pango_layout_get_cursor_pos (display->layout, index,
                                strong_pos ? &pango_strong_pos : NULL,
                                weak_pos ? &pango_weak_pos : NULL);
@@ -2684,6 +2731,9 @@ gtk_text_layout_move_iter_to_next_line (GtkTextLayout *layout,
       line = _gtk_text_line_next_excluding_last (line);
     }
 
+  if (!found_after)
+    gtk_text_buffer_get_end_iter (layout->buffer, iter);
+  
   return
     !gtk_text_iter_equal (iter, &orig) &&
     !gtk_text_iter_is_end (iter);
