@@ -176,6 +176,9 @@ struct _GtkTreeViewPrivate
   gint press_start_x;
   gint press_start_y;
 
+  /* fixed height */
+  gint fixed_height;
+
   /* Scroll-to functionality when unrealized */
   GtkTreeRowReference *scroll_to_path;
   GtkTreeViewColumn *scroll_to_column;
@@ -183,6 +186,7 @@ struct _GtkTreeViewPrivate
   gfloat scroll_to_col_align;
   guint scroll_to_use_align : 1;
 
+  guint fixed_height_mode : 1;
   guint fixed_height_check : 1;
 
   guint reorderable : 1;
@@ -192,6 +196,9 @@ struct _GtkTreeViewPrivate
   guint has_rules : 1;
   guint mark_rows_col_dirty : 1;
 
+  /* for DnD */
+  guint empty_view_drop : 1;
+
   guint ctrl_pressed : 1;
   guint shift_pressed : 1;
   
@@ -203,21 +210,8 @@ struct _GtkTreeViewPrivate
   GtkTreeViewSearchEqualFunc search_equal_func;
   gpointer search_user_data;
   GtkDestroyNotify search_destroy;
-};
-
-/* cool ABI compat hack */
-#define GTK_CELL_RENDERER_INFO_KEY "gtk-cell-renderer-info"
-
-typedef struct _GtkCellRendererInfo GtkCellRendererInfo;
-struct _GtkCellRendererInfo
-{
-  GdkColor cell_background;
-
-  /* text renderer */
-  gulong focus_out_id;
-
-  /* toggle renderer */
-  guint inconsistent :1;
+  guint init_hadjust_value :1;              
+  gint prev_width;                          
 };
 
 #ifdef __GNUC__
